#use strict;
use warnings;
use diagnostics;

use ChIPSeqPipeline_sample;

package Group;

sub new {
  my $self  = {};
  $self ->{STUDY} = undef;
  $self ->{NAME} = undef;
  $self ->{SAMPLES} = [];
  bless($self); 
  return $self;
}

sub study{
  my $self = shift;
  if (@_) { $self->{STUDY}  = shift; }
  return $self->{STUDY};
}
sub name{
  my $self = shift;
  if (@_) { $self->{NAME}  = shift; }
  return $self->{NAME};
}
sub samples{
  my $self = shift;
  if(@_){ push@{ $self->{SAMPLES} }, @_ }
  return @{ $self->{SAMPLES} };
}

sub samplesNum{
  my $self = shift;
  return ( $#{ $self->{SAMPLES} } +1 );
}

sub isEqual{
  ### Compare two group objects to determine similarity.
  ### Comparison is based on group name only (at this point).
  my $self  = shift;
  if( !(@_) ){ print"Error: isEqual got an empty group object\n"; }
  my($groupObjct) = @_;
  my $selfName = $self -> name(); 
  my $groupObjctName = $groupObjct -> name();
  if(  $selfName eq $groupObjctName ){ return 1; }
  return 0;
}

sub addSample{
 ### Add a sample object to group's samples array if sample is new.
  ### Samples are compared using the isEqual subroutine of the Sample package.
  my $self = shift;
  my($sampleObjct) = @_;
  if( ( $self -> isNewSample( $sampleObjct ) ) ==1 ){
    $self->samples( $sampleObjct );
    return 1;
  }
  return 0;
}

sub isNewSample{
  ### Returns 1 if sample is new and zero otherwise.
  my $self = shift;
  my($sampleObjct) = @_;
  foreach my $sample ( $self -> samples() ){
    if( $self->isEqual( $sampleObjct ) == 1 ){ return 0; }
  }
  return 1;
}

sub print{
  my $self = shift;
  my $name = $self -> name(); my $samplesNum = $self -> samplesNum();
  print"\t\t\t~~~~~~~~~~ Group $name has $samplesNum samples ~~~~~~~~~~~\n";
  foreach my $sample ( $self -> samples() ){
    $sample->print();
  }
}

1;
