#use strict;
use warnings;
use diagnostics;

package Input;

my @arrGeneralKeys = ("g1","g2");
my @arrSpecificKeys = ("s1", "s2");

sub new{
  my $self  = {};
  bless($self); 
  return $self;
}

sub usage{
  print"Usage: $0 \n\n\t[-g] [-l|-d]\n\t-g general parameter file\n\t-l study specific parameter file list\n\t-d directory of study specific parameter files\n\t -fg general param file format\t-fs study param file format\nNote: At least one general file and one specific file must be provided.\n\nExample: perl $0  -g /root/downloads/SequenceCentric/ChIPSeq/code/pipeline/ChIPSeqPipeline_generalParams.txt -l /root/downloads/SequenceCentric/ChIPSeq/code/pipeline/ChIPSeqPipeline_studySpecificParams.txt \n\nRequired general parameters\n@arrGeneralKeys\nRequired specific parameters\n@arrSpecificKeys\n\nWorking example: perl ChIPSeqPipeline_main.pl -d /root/downloads/SequenceCentric/ChIPSeq/code/pipeline/test/ -g ChIPSeqPipeline_generalParams.txt\n"; 
  exit;
}


sub commandLineParams{
  my $self  = shift;
  ### Process parameters from command line. Input is ARGV array.
  my ($refHashParams, @inputArr ) = @_;
  if ( @inputArr < 1 ) { usage(); }
  while(scalar(@inputArr) > 0){
    my $this_arg = shift @inputArr;
    print"$this_arg\n";
    if ( $this_arg eq '-h') { usage(); }
    ### General param file
    elsif ( $this_arg eq '-g'  ) {
      my $general_paramFile = shift @inputArr;
      #print "$general_paramFile\n";
      $self -> hashParams( $general_paramFile, $refHashParams );
    }
    ### Study specific param file list
    elsif ( $this_arg eq '-l' ) {
      @{$$refHashParams{ "STUDY_PARAM_FILE_LIST" }} = @inputArr;
    }
    ### directory with study specific param files
    elsif ( $this_arg eq '-d') {
      if( @inputArr ){
	my $specifcParamDir = shift @inputArr;
	$$refHashParams{ "STUDY_PARAM_FILE_DIR" } = $specifcParamDir;
	print" Adding STUDY_PARAM_FILE_DIR $specifcParamDir\n";
      }
      else{ print"Error: A directory name is not specified after -d tag\n"; }
    }
    ### Format of general param file
    elsif ( $this_arg eq '-fg') {
      ### TO BE COMPLETED!!! 
    }
    ### Format of study specific param file
    elsif ( $this_arg eq '-fs') {
      ### TO BE COMPLETED!!! 
    }
    elsif ( $this_arg =~ m/^-/ ) { print "unknown flag: $this_arg\n"; }
    #else ( $this_arg =~ m/^-/ ) { print "unknown argument: $this_arg\n"; }
  }
}


sub hashParams{
  ### Hash parameters from an input file. Expects key and values in each line.
  ### Similar keys can overwrite one another ($isConcat = NO_CONCAT) or concatenate with a delimiter.
  ### NO_CONCAT is optional 
  my $self  = shift;
  my ($paramFile, $hashRef, $isConcat )= @_;
  my $delimiterSign = $self -> delimiterDetection_File( $paramFile, "\t", "#", 2, 10);
  #print"delimiterSign [$delimiterSign]\n";
  open PARAMFILE, "<","$paramFile" or die "ERROR: hashParams: Could not open param file $paramFile";
  while (my $line = <PARAMFILE>){
    if( $line =~ /^#/ ){ next; }### Skip comment lines
    if( length($line) < 2 ){ next; }### Skip empty lines
    chomp $line;
    my @lineArr =  split( /$delimiterSign/, $line );

    my $key = shift(@lineArr);
    foreach my $val (@lineArr ){
      if( !(defined $key) || !(defined($val) ) ){ next; }
      $val =~ s/\s+$//; $val =~ s/^\s+//; ###Remove trailing and leading spaces
      #print"$line\nkey [$key] val [$val]\n";
      ### Case first encounter of key - hash val by key
      if( ! defined($$hashRef{ $key }) ){
	$$hashRef{ $key } = $val;
      }
      ### Case key was already used concat the val using a delimiter sign.
      else{ 
	if( ! defined($isConcat) ){###Default is concat similar 
	  $$hashRef{ $key } = $$hashRef{ $key }."$delimiterSign"."$val";
	}
	elsif( $isConcat eq "NO_CONCAT" ){
	  $$hashRef{ $key } = $val;
	}
	else{
	  $$hashRef{ $key } = $$hashRef{ $key }."$delimiterSign"."$val";
	}
      }
    }
  }
  close(PARAMFILE)
}

sub delimiterDetection_File{
  my $self  = shift;
  my ($inFile, $expectedDelimiterSign, $commentSign, $minLineLength, $maxLines)= @_;
  #print"--------Detecting delimiter for $inFile\n";
  
  #print"@seperatorArr [@seperatorArr]\n";
  my $lineCnt = 0; my $seperator = "";  my @noSepLineArr = ();
  my %hashSepCnt = (); ### KEY = seperator. VAL = Line Count.
  open INFILE, "<","$inFile" or die "ERROR: Could not open file $inFile";
  ### Need  to add an exception handle to check if file exists.
  while (my $line = <INFILE>){
    if( $lineCnt >= $maxLines ){ last; }
    if( $line =~ /^$commentSign/ ){ next; }### Skip comment lines
    if( length($line) <  $minLineLength){ next; }### Skip short (typically empty) lines
    my $seperator = $self -> delimiterDetection_Line( $line, $expectedDelimiterSign );
    ### Case seperator is detected in line
    if( length($seperator) > -1 ){
      $hashSepCnt{ $seperator }++;
    }
    else{### Case no seperator is detected
      push( @noSepLineArr, "$line\n" );
    }
    ++$lineCnt;
  }
  #print "size of hashSepCnt:  " . keys( %hashSepCnt ) . ".\n";
  my @sortedKeys = sort {$hashSepCnt{$b} <=>  $hashSepCnt{$a}} keys %hashSepCnt;
  my $finalSeperator = $sortedKeys[ 0 ];
  #print"finalSeperator [$finalSeperator] sortedKeys [@sortedKeys]\n";
  if( ! defined($hashSepCnt{ $finalSeperator }) ){
    print"Could not detect delimiter in file [$inFile]\n"; exit;
  }
  my $sepCnt = $hashSepCnt{ $finalSeperator }; 
  #print"finalSeperator [$finalSeperator] sepCnt [$sepCnt] lineCnt $lineCnt\n";
  my $percentLines = ( $hashSepCnt{ $finalSeperator } / $lineCnt ) *100;
  #### print"$percentLines % of lines in $inFile are seperated by [$finalSeperator]\n";
  #if( $#noSepLineArr > 0 ){
   # print"Example lines without proper [$finalSeperator] seperation:\n@noSepLineArr\n";
  #}
  #print"------------ Finished delimiter detection------------------\n";
  return $finalSeperator;
}


sub delimiterDetection_Line{
  ### Returns the seperator or an empty string
  my $self  = shift;
  my ($line, $expectedDelimiterSign )= @_;
  my @seperatorArr = ("$expectedDelimiterSign","\t",",",";","\\\|"," ");
  chomp $line;  
  my $seperator = "";
  foreach $seperator (@seperatorArr){
    #print"~~~~~~ Start seperator [$seperator] ~~~~~~ \n";
    if( $line =~ /$seperator/ ){ 
      #print"$line\nseperator is [$seperator]\n";
      my @splitArr =  split( /$seperator/, $line );
      #$a=$#splitArr; print"size is $a\n";
      if( $#splitArr >= 1 ){
	return $seperator;
      }
    }
  }
  return "";
}



1;
