package StudyMngr;

#use strict;
use warnings;
use diagnostics;

use ChIPSeqPipeline_processStepMngr;



sub new {
  my $self  = {};
  $self->{STUDY_PARAM_FILES} = [];
  $self->{GENERAL_PARAMS} = undef;
  $self->{STUDIES} = [];
#  $self->{PROCESS_STUDIES} = [];
#  $self->{FAILED_STUDIES} = [];
#  $self->{FINISHED_STUDIES} = [];
  bless($self); 
  return $self;
}

sub studies{
  my $self = shift;
  if(@_){ push@{ $self->{STUDIES} }, @_ }
  return @{ $self->{STUDIES} };
}

sub studiesNum{
  my $self = shift;
  return ( $#{ $self->{STUDIES} } + 1 );
}

sub addStudyParamFiles{
  my $self = shift;
  if(@_){ push@{ $self->{STUDY_PARAM_FILE_LIST} }, @_ }
  return @{ $self->{STUDY_PARAM_FILE_LIST} };
}

sub generalParams{
  my $self = shift;
  if (@_) { $self->{GENERAL_PARAMS} = shift }
  return $self->{GENERAL_PARAMS};
}

sub getGeneralParam{
  my $self = shift;
  my $refHash = $self->{GENERAL_PARAMS};
  my $key = shift;
  return $$refHash{ $key };
}

sub printGeneralParams{
  my $self = shift;
  my $refHash = $self->{GENERAL_PARAMS};
  if( !(defined($refHash)) ){ print"Error: undefined hash reference\n"; }
  print"------- General parameters --------------\n";
  print "size of hash:  " . keys( %$refHash ) . ".\n";
  foreach my $key ( sort( keys(%$refHash) ) ){
    my $val = $$refHash{ $key };
    print"[$key] -> [$val]\n";
  }
  print"------- End general parameters ----------\n";
}

sub process{
  my $self = shift;
  $self -> newStudyParamFiles2StudyObjct();
  ### Iterate over studies and create process step managers
  foreach my $studyObjct ( $self -> studies() ){
    ### Create a processStepMngr  
###HERE !!!!
    my $processStepMngr = new ProcessStepMngr();
    $processStepMngr -> generalParams( $self -> generalParams() );
    $processStepMngr -> continueProcess();
    ###
  }
    
}

sub newStudyParamFiles2StudyObjct{
  ###Go over old study param file list (if exists) and compares with current input of study param files.
  ### For new or revised files a study object is generated.
  my $self = shift;
  my @studiesList = $self -> studyParamFiles_fromStudies();
  my @inputList = $self -> studyParamFiles_fromInput(); 
  #my $l1 = $#studiesList; my $l2 = $#inputList; print"$l1 $l2\n";
  ### create a new Study object for new study specific param files encountered.
  my $studyObjct; my $newCntr =0; my $revisedCntr = 0;
  foreach my $inputStudyParamFile ( @inputList ){
    my $paramFileStatus = paramFileStatus( $inputStudyParamFile, \@studiesList );
    print"file $inputStudyParamFile status [$paramFileStatus] \n";
    if( $paramFileStatus =~ m/NEW/ ){
      $studyObjct = Study -> new;
      $studyObjct -> paramFile( $inputStudyParamFile );### This also processes parameter file
      $studyObjct = ProcessStepMngr -> new; 
      $self -> studies( $studyObjct );
      ++$newCntr;
    }
    elsif( $paramFileStatus =~ m/REVISED/ ){
      ### TO BE COMPLETE !!!! delete study from failed list before restarting it
      $studyObjct = Study->new;
      $studyObjct -> paramFile( $inputStudyParamFile );### This also processes parameter file
      $studyObjct -> studyMngr( $self );
      ++$revisedCntr;
    }
  }
  print"Initiating $newCntr new and $revisedCntr revised studies\n";
}


sub studyParamFiles_fromStudies{
  my $self = shift;
  ### Iterates over studies and get study param file names
  my @list =();
  foreach my $studyRef ( @{ $self->{STUDY_PARAM_FILE_LIST} } ){
    push( @list, $studyRef -> paramFile() );
  }
  return @list;
}

sub studyParamFiles_fromInput{
  my $self = shift;
  ### If a directory was specified read files from this dir. These are added to list. Otherwise 
  my @fileList = ();
  my $refHash = $self -> {GENERAL_PARAMS};
  my $dir = $$refHash{ "STUDY_PARAM_FILE_DIR" };
  if(defined ($dir) ){
    #print"defined dir [$dir]\n";
    @fileList = dir2list( $dir );
  }
  #print"fileList before [@fileList]\n";
  if( defined(@{$refHash->{"STUDY_PARAM_FILE_LIST"}}) ){
    @fileList = (@fileList, @{$refHash->{"STUDY_PARAM_FILE_LIST"}});#Concatanate array
  }
  #print"fileList after  [@fileList]\n";
  uniqArray( \@fileList );
  return @fileList;
}

sub paramFileStatus{
  my ($paramFile, $refList) = @_;
  foreach my $currFile (@$refList){
    if( $currFile eq $paramFile ){ return "OLD"; }
    elsif( $currFile eq "revised_"."$paramFile" ){ return "REVISED"; }
  }
  return "NEW";
}

sub uniqArray {  
  my ($arrRef) = @_; my %hash = ();
  my @arr = @{$arrRef};
  if( $#arr == 0 ){ return; }
  foreach my $element ( @arr ) {
    $hash{ $element } = "1";
  }
  @$arrRef = sort keys( %hash );
  return $arrRef;
  ### Make an array unique
  @_ = grep {!$hash{$_}++} @_;
  #return keys %{{ map { $_ => 1 } @_ }};
}

sub dir2list{
  ### This subroutine gets a directory and returns a list of files in this dir. An optional parameter is an expected extension of file name.
  my ($dir, $extension) = @_;
  if (!(-d "$dir")){ print "Dir [$dir] does not exist.\nPlease check path or provide a list of files with -l flag\n"; exit; }
  my @list =();
  opendir DIR, "$dir" || die "Cannot open directory $dir";
  while ( my $filename = readdir(DIR) ) {
    if( length( $filename ) <= 2 ){ next; }
    if( defined ($extension) ){
      if( $filename !~ m/$extension$/ ){ next; }
    }	
    push( @list, "$dir"."$filename" );
  }
  return @list;
}



sub print{
  my $self = shift;
  my $studyNum = $self->studiesNum();
  print"~~~~~~~~~~ Study Manager has $studyNum studies ~~~~~~~~~~\n";
  foreach my $study ( $self-> studies() ){
    $study -> print();
  }
}


1;
